@@ -6,6 +6,7 @@ Notify thread to handle change notifications
 from datetime import datetime, timedelta
 import requests
 import socket
+import sys
 import threading
 import time
 from ihcsdk.ihcclient import IHCSoapClient, IHCSTATE_READY
@@ -47,6 +48,7 @@ class IHCController:
                 return False
             return True
         except requests.exceptions.RequestException as exp:
+            print("is_ihc_controller: %s" % exp, file=sys.stderr)
             return False
 
     def authenticate(self) -> bool:
@@ -133,7 +135,7 @@ class IHCController:
                     ready = self.client.wait_for_state_change(IHCSTATE_READY, 10)
                     if ready != IHCSTATE_READY:
                         return None
-                self._project = self.client.get_project()
+                self._project = self.client.get_project_in_segments()
         return self._project
 
     def add_notify_event(self, resourceid: int, callback, delayed=False):
